﻿/*
 * Author: Adrian Posor
 * Year: 2009
 * License: GPL
 */
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.IO;
using System.Windows.Forms;
using System.Net;
using System.Net.Sockets;
using RDPCOMAPILib;
using AxRDPCOMAPILib;

namespace Desktop_Sharing_2009
{
    public partial class MainWindow : Form
    {
        RDPSession SharingSession = null;
        AxRDPViewer Viewer = null;
        IRDPSRAPIInvitation invitation_from_server = null;
        IRDPSRAPISessionProperties ServerSessionProperties = null;
        IRDPSRAPISessionProperties ClientSessionProperties = null;
        //String reverse_connection_string = null;
        //String connection_string = null;
        bool reverseConnectionListenerStarted = false;
        bool settingsLoaded = false;
        int[] SharingSessionRectangle = new int[4];
        Form2 ViewerWindow = null;
        Point lastViewerWindowLocation = new Point();
        Size lastViewerwindowSize = new Size();



        public MainWindow()
        {
            InitializeComponent();
            write = new FunctionPointer(writeToTextBox);
            ViewerWindow = new Form2(this);
            Viewer = ViewerWindow.getViewer();
            ClientSessionProperties = Viewer.Properties;
            Control.CheckForIllegalCrossThreadCalls = false;
            numericUpDown_screen_number.Maximum = Screen.AllScreens.Length;
            numericUpDown_screen_number2.Maximum = Screen.AllScreens.Length;

            loadSettings();

            if (Screen.AllScreens.Length < 2)
            {
                checkBox_allow_multi_monitor_fullscreen.Enabled = false;
                checkBox_allow_multi_monitor_fullscreen.Checked = false;
            }

            startClientSocketListener();

            if (checkBox_client_set_rpd_port.Checked)
            {
                ClientSessionProperties["PortId"] = Convert.ToInt32(textBox_client_RDP_port.Text);
            }
        }

        private void server_connect_via_ip_button_Click(object sender, EventArgs e)
        {
            if (SharingSession == null)
            {
                textBox1.Text += Environment.NewLine + "Please start the server first.";
                return;
            }

            textBox1.Text += Environment.NewLine + "Trying to connect via IP...";

            try
            {
                ServerConnectViaIPSocket = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);
                ServerConnectViaIPSocket.Connect(new IPEndPoint(IPAddress.Parse(textBox_server_connect_ip.Text), Convert.ToInt32(textBox_server_connect_port.Text)));
                ServerConnectViaIPSocket.Send(System.Text.Encoding.ASCII.GetBytes(invitation_from_server.ConnectionString));

                CSocketPacket theSocPkt = new CSocketPacket();
                theSocPkt.thisSocket = ServerConnectViaIPSocket;
                // now start to listen for any data...
                ServerConnectViaIPSocket.BeginReceive(theSocPkt.dataBuffer, 0, theSocPkt.dataBuffer.Length, SocketFlags.None, OnDataReceivedServer, theSocPkt);
            }
            catch (Exception ex)
            {
                textBox1.Text += Environment.NewLine + "The server could not connect to the client because the following exception occurred: " + ex.ToString();
            }
        }

        private void client_connect_via_ip_button_Click(object sender, EventArgs e)
        {
            textBox1.Text += Environment.NewLine + "Trying to connect via IP...";
            viewerDisableConnectButtons();

            try
            {
                ClientSocketListener.Close();
            }
            catch (ObjectDisposedException) { }
            catch (SocketException) { }
            catch (NullReferenceException) { }

            try
            {
                ClientConnectViaIPSocket = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);
                ClientConnectViaIPSocket.Connect(new IPEndPoint(IPAddress.Parse(textBox_viewer_connect_ip.Text), Convert.ToInt32(textBox_viewer_connect_port.Text)));
                ClientConnectViaIPSocket.Send(System.Text.Encoding.ASCII.GetBytes("RI"));

                //byte[] myBuffer = new byte[500];
                //ClientConnectViaIPSocket.Receive(myBuffer);

                CSocketPacket theSocPkt = new CSocketPacket();
                theSocPkt.thisSocket = ClientConnectViaIPSocket;
                // now start to listen for any data...
                ClientConnectViaIPSocket.BeginReceive(theSocPkt.dataBuffer, 0, theSocPkt.dataBuffer.Length, SocketFlags.None, OnDataReceivedViewer, theSocPkt);
            }
            catch (Exception ex)
            {
                textBox1.Text += Environment.NewLine + "The client could not connect to the server because the following exception occurred: " + ex.ToString();
                viewerEnableConnectButtons();
            }
        }

        private void create_invitation_file_button_Click(object sender, EventArgs e)
        {
            if (SharingSession == null)
            {
                textBox1.Text += Environment.NewLine + "Please start the server first.";
                return;
            }

            saveFileDialog1.Filter = "invitation files (*.xml)|*.xml";
            saveFileDialog1.RestoreDirectory = true;

            if (saveFileDialog1.ShowDialog() == DialogResult.OK)
            {
                using (StreamWriter myStream = new StreamWriter(saveFileDialog1.FileName))
                {
                    // Code to write the stream goes here.
                    myStream.Write(invitation_from_server.ConnectionString);
                    myStream.Close();
                }
            }

        }

        private void server_connect_via_connection_string_button_Click(object sender, EventArgs e)
        {
            if (SharingSession == null)
            {
                textBox1.Text += Environment.NewLine + "Please start the server first.";
                return;
            }

            try
            {
                SharingSession.ConnectToClient(textBox_connection_string_server.Text);
            }
            catch (Exception ex)
            {
                textBox1.Text += Environment.NewLine + "The server could not connect to the client because the following exception occurred: " + ex.ToString();
            }
        }



        private void client_connect_via_invitation_file_button_Click(object sender, EventArgs e)
        {
            String connection_string;
            if (!File.Exists(textBox_file_name.Text))
            {
                textBox1.Text += Environment.NewLine + "Please specify a valid path and file name.";
                return;
            }
            using (StreamReader myStream = new StreamReader(textBox_file_name.Text))
            {
                connection_string = myStream.ReadToEnd();
                myStream.Close();
            }
            client_connect(connection_string, true);
        }

        private void client_connect(String connection_string, bool handleException)
        {
            startViewer();
            viewerDisableConnectButtons();

            try
            {
                String name;
                if (textBox_user_name.TextLength == 0)
                {
                    name = Environment.UserName + "@" + Environment.MachineName;
                }
                else
                {
                    name = textBox_user_name.Text;
                }

                Viewer.Connect(connection_string, name, "");
            }
            catch (Exception ex)
            {
                if (handleException)
                {
                    viewerEnableConnectButtons();
                    textBox1.Text += Environment.NewLine + "The client could not connect to the server because the following exception occurred: " + ex.ToString();
                }
                else
                {
                    throw ex;
                }
            }
        }

        private void client_disconnect_button_Click(object sender, EventArgs e)
        {
            if (Viewer != null)
            {
                try
                {
                    foreach (IRDPSRAPIAttendee pAttendee in Viewer.Attendees)
                    {
                        listBox_participants.Items.Remove(pAttendee.RemoteName + " (" + pAttendee.Id + ")");
                    }

                    Viewer.Disconnect();
                }
                catch (Exception ex)
                {
                    textBox1.Text += "The connection could not be terminated because the following exception occurred: " + ex.ToString();
                }
            }


            viewerEnableConnectButtons();
        }


        private void openFileDialog1_FileOk(object sender, CancelEventArgs e)
        {

        }

        private void saveFileDialog1_FileOk(object sender, CancelEventArgs e)
        {

        }


        private void client_choose_invitation_file_button_Click(object sender, EventArgs e)
        {
            openFileDialog1.Filter = "invitation files (*.xml)|*.xml";


            if (openFileDialog1.ShowDialog() == DialogResult.OK)
            {
                textBox_file_name.Text = openFileDialog1.FileName;
            }

        }

        private void start_server_button_Click(object sender, EventArgs e)
        {
            if (SharingSession == null)
            {
                SharingSession = new RDPSession();
            }
            textBox1.Text += Environment.NewLine + "New RDP Session created.";

            ServerSessionProperties = SharingSession.Properties;
            if (radioButton_mirror_driver_attachment_dynamic.Checked == true)
            {
                ServerSessionProperties["DrvConAttach"] = true;
            }
            else if (radioButton_mirror_driver_attachment_static.Checked == true)
            {
                ServerSessionProperties["DrvConAttach"] = false;
            }

            if (checkBox_server_set_rdp_port.Checked == true)
            {
                ServerSessionProperties["PortId"] = Convert.ToInt32(textBox_server_RDP_port.Text);
            }

            try
            {
                SharingSession.Open();
                SharingSession.OnAttendeeConnected += new RDPCOMAPILib._IRDPSessionEvents_OnAttendeeConnectedEventHandler(OnAttendeeConnected);
                SharingSession.OnAttendeeDisconnected += new RDPCOMAPILib._IRDPSessionEvents_OnAttendeeDisconnectedEventHandler(OnAttendeeDisconnected);
                SharingSession.OnControlLevelChangeRequest += new RDPCOMAPILib._IRDPSessionEvents_OnControlLevelChangeRequestEventHandler(OnControlLevelChangeRequest);
                SharingSession.OnConnectionFailed += new RDPCOMAPILib._IRDPSessionEvents_OnConnectionFailedEventHandler(OnConnectionToClientFailed);
                SharingSession.OnError += new RDPCOMAPILib._IRDPSessionEvents_OnErrorEventHandler(OnSharingSessionError);
                invitation_from_server = SharingSession.Invitations.CreateInvitation("Desktop Sharing 2009", "workgroup", "", Convert.ToInt32(textBox_server_max_attendees.Text));

                startServerSocketListener();

                SharingSession.GetDesktopSharedRect(out SharingSessionRectangle[0], out SharingSessionRectangle[1], out SharingSessionRectangle[2], out SharingSessionRectangle[3]);
                textBox_sharing_rectangle_left.Text = SharingSessionRectangle[0].ToString();
                textBox_sharing_rectangle_top.Text = SharingSessionRectangle[1].ToString();
                textBox_sharing_rectangle_right.Text = SharingSessionRectangle[2].ToString();
                textBox_sharing_rectangle_bottom.Text = SharingSessionRectangle[3].ToString();

                start_server_button.Enabled = false;
                pause_server_button.Enabled = true;
                resume_server_button.Enabled = false;
                stop_server_button.Enabled = true;

                foreach (ToolStripItem item in contextMenuStrip_participants.Items)
                {
                    item.Enabled = true;
                }

                textBox1.Text += Environment.NewLine + "RDP Server started.";
            }
            catch (Exception ex)
            {
                textBox1.Text += Environment.NewLine + "The RDP server could not be started because the following exception occurred: " + ex.ToString();
            }
        }

        private void pause_server_button_Click(object sender, EventArgs e)
        {
            SharingSession.Pause();
            pause_server_button.Enabled = false;
            resume_server_button.Enabled = true;
        }

        private void resume_server_button_Click(object sender, EventArgs e)
        {
            SharingSession.Resume();
            pause_server_button.Enabled = true;
            resume_server_button.Enabled = false;
        }

        private void stop_server_button_Click(object sender, EventArgs e)
        {
            start_server_button.Enabled = true;
            pause_server_button.Enabled = false;
            resume_server_button.Enabled = false;
            stop_server_button.Enabled = false;

            try
            {
                foreach (IRDPSRAPIAttendee pAttendee in SharingSession.Attendees)
                {
                    listBox_participants.Items.Remove(pAttendee.RemoteName + " (" + pAttendee.Id + ")");
                }

                SharingSession.Close();
                textBox1.Text += Environment.NewLine + "RDP server stopped." + Environment.NewLine;
                System.Runtime.InteropServices.Marshal.ReleaseComObject(SharingSession);
                SharingSession = null;

                start_server_button.Enabled = true;
                pause_server_button.Enabled = false;
                resume_server_button.Enabled = false;
                stop_server_button.Enabled = false;

                try
                {
                    ServerSocketListener.Close();
                }
                catch (ObjectDisposedException) { }
                catch (NullReferenceException) { }
                catch (SocketException) { }

                foreach (ToolStripItem item in contextMenuStrip_participants.Items)
                {
                    item.Enabled = false;
                }
            }
            catch (Exception ex)
            {
                textBox1.Text += "Error occured while stopping the RDP server. Error: " + ex.ToString();
            }
        }

        private void server_copy_connection_string_button_Click(object sender, EventArgs e)
        {
            if (invitation_from_server != null)
            {
                //textBox_connection_string_server.Text = invitation_from_server.ConnectionString;
                Clipboard.SetDataObject(invitation_from_server.ConnectionString);
            }
        }

        private void client_connect_via_connection_string_button_Click(object sender, EventArgs e)
        {
            client_connect(textBox_connection_string_client.Text, false);
        }


        private void viewer_create_connection_string_button_Click(object sender, EventArgs e)
        {
            String reverse_connection_string;
            if (textBox_connection_string_client.Text.Length == 0)
            {
                textBox1.Text += Environment.NewLine + "Connection string from server is needed for reverse connection.";
                return;
            }
            startViewer();
            reverse_connection_string = start_client_listener(textBox_connection_string_client.Text);
            textBox_reverse_connection_string.Text = reverse_connection_string;
            Clipboard.SetDataObject(reverse_connection_string);
        }

        // Server events
        private void OnAttendeeConnected(object pObjAttendee)
        {
            IRDPSRAPIAttendee pAttendee = pObjAttendee as IRDPSRAPIAttendee;
            pAttendee.ControlLevel = CTRL_LEVEL.CTRL_LEVEL_VIEW;
            textBox1.Text += Environment.NewLine + ("Attendee connected: " + pAttendee.RemoteName);
            listBox_participants.Items.Add(pAttendee.RemoteName + " (" + pAttendee.Id + ")");

            try
            {
                ServerConnectViaIPSocket.Shutdown(SocketShutdown.Both);
                ServerConnectViaIPSocket.Close();
            }
            catch (ObjectDisposedException) { }
        }

        private void OnAttendeeDisconnected(object pDisconnectInfo)
        {
            IRDPSRAPIAttendeeDisconnectInfo pDiscInfo = pDisconnectInfo as IRDPSRAPIAttendeeDisconnectInfo;
            textBox1.Text += Environment.NewLine + ("Attendee disconnected: " + pDiscInfo.Attendee.RemoteName);
            listBox_participants.Items.Remove(pDiscInfo.Attendee.RemoteName + " (" + pDiscInfo.Attendee.Id + ")");
        }

        private void OnControlLevelChangeRequest(object pObjAttendee, CTRL_LEVEL RequestedLevel)
        {
            IRDPSRAPIAttendee pAttendee = pObjAttendee as IRDPSRAPIAttendee;

            if (RequestedLevel == CTRL_LEVEL.CTRL_LEVEL_VIEW)
            {
                pAttendee.ControlLevel = RequestedLevel;
                textBox1.Text += Environment.NewLine + "Set control level of attendee " + pAttendee.RemoteName + " to " + RequestedLevel.ToString();
                return;
            }

            if (radioButton_always_allow.Checked == true)
            {
                pAttendee.ControlLevel = RequestedLevel;
                textBox1.Text += Environment.NewLine + "Set control level of attendee " + pAttendee.RemoteName + " to " + RequestedLevel.ToString();
            }
            else if (radioButton_always_forbid.Checked == true)
            {
                // do nothing
            }
            else if (radioButton_ask_before.Checked == true)
            {
                string message = "Attendee " + pAttendee.RemoteName + " would like to control your desktop. Do you want to allow this?";
                string caption = "Desktop Sharing 2009";
                MessageBoxButtons buttons = MessageBoxButtons.YesNo;
                DialogResult result;

                // Displays the MessageBox.

                result = MessageBox.Show(this, message, caption, buttons,
                MessageBoxIcon.Information, MessageBoxDefaultButton.Button1);

                if (result == DialogResult.Yes)
                {
                    pAttendee.ControlLevel = RequestedLevel;
                    textBox1.Text += Environment.NewLine + "Set control level of attendee " + pAttendee.RemoteName + " to " + RequestedLevel.ToString();
                }
            }
        }

        private void OnConnectionToClientFailed()
        {
            textBox1.Text += Environment.NewLine + "Connection to client failed.";
        }

        private void OnSharingSessionError(object ErrorInfo)
        {
            textBox1.Text += Environment.NewLine + "The following error occured: " + ErrorInfo.ToString();
        }
        // end of server events

        // my own methods
        private String start_client_listener(String connectionString)
        {
            String name;
            String reverse_connection_string = null;
            if (textBox_user_name.TextLength == 0)
            {
                name = Environment.UserName + "@" + Environment.MachineName;
            }
            else
            {
                name = textBox_user_name.Text;
            }

            try
            {
                reverse_connection_string = Viewer.StartReverseConnectListener(connectionString, name, "");
                reverseConnectionListenerStarted = true;
            }
            catch (Exception ex)
            {
                textBox1.Text += Environment.NewLine + "The reverse connect listener could not be started because the following exception occurred: " + ex.ToString();
                reverseConnectionListenerStarted = false;
            }
            return reverse_connection_string;
        }

        private void startViewer()
        {
            if (Viewer == null)
            {
                Viewer = ViewerWindow.getViewer();
            }
            ViewerWindow.Show();
        }

        private void viewerDisableConnectButtons()
        {
            client_connect_via_ip_button.Enabled = false;
            client_connect_via_connection_string_button.Enabled = false;
            client_connect_via_invitation_file_button.Enabled = false;
        }

        public void viewerEnableConnectButtons()
        {
            client_connect_via_ip_button.Enabled = true;
            client_connect_via_connection_string_button.Enabled = true;
            client_connect_via_invitation_file_button.Enabled = true;
        }

        private void saveSettings()
        {
            try
            {
                FileStream fs = File.Open("ds09.cfg", FileMode.Create);

                using (BinaryWriter bw = new BinaryWriter(fs))
                {
                    // server tab
                    bw.Write(textBox_server_connect_port.Text);
                    bw.Write(radioButton_always_allow.Checked);
                    bw.Write(radioButton_always_forbid.Checked);
                    bw.Write(radioButton_ask_before.Checked);
                    // viewer tab
                    bw.Write(textBox_viewer_connect_port.Text);
                    //server settings tab
                    bw.Write(checkBox_server_enable_invitation_requests.Checked);
                    bw.Write(textBox_server_listen_connection_via_ip_port.Text);
                    bw.Write(checkBox_server_set_rdp_port.Checked);
                    bw.Write(textBox_server_RDP_port.Text);
                    bw.Write(textBox_server_max_attendees.Text);
                    bw.Write(radioButton_mirror_driver_attachment_dynamic.Checked);
                    bw.Write(radioButton_mirror_driver_attachment_static.Checked);
                    // viewer settings tab
                    bw.Write(checkBox_client_listen_for_invitations.Checked);
                    bw.Write(textBox_client_listen_connection_via_ip_port.Text);
                    bw.Write(checkBox_client_set_rpd_port.Checked);
                    bw.Write(textBox_client_RDP_port.Text);
                    bw.Write(checkBox_allow_multi_monitor_fullscreen.Checked);
                    // global settings
                    bw.Write(textBox_user_name.Text);
                    bw.Write(checkBox_save_settings_on_exit.Checked);

                    bw.Close();
                }
            }
            catch (UnauthorizedAccessException)
            {
                string message = "The settings could not be saved to a file because of missing permissions. Please run this program as an administrator or change the permissions for this directory.";
                string caption = "Desktop Sharing 2009";
                MessageBoxButtons buttons = MessageBoxButtons.OK;


                // Displays the MessageBox.

                MessageBox.Show(this, message, caption, buttons,
                MessageBoxIcon.Error, MessageBoxDefaultButton.Button1);
            }
            catch (IOException e)
            {
                string message = "The following error occured while saving the settings: " + e.Message;
                string caption = "Desktop Sharing 2009";
                MessageBoxButtons buttons = MessageBoxButtons.OK;


                // Displays the MessageBox.

                MessageBox.Show(this, message, caption, buttons,
                MessageBoxIcon.Error, MessageBoxDefaultButton.Button1);
            }
        }

        private void loadSettings()
        {
            if (File.Exists("ds09.cfg"))
            {
                FileStream fs = null;
                BinaryReader br = null;
                try
                {
                    fs = File.Open("ds09.cfg", FileMode.Open);
                    br = new BinaryReader(fs);
                    // server tab
                    textBox_server_connect_port.Text = br.ReadString();
                    radioButton_always_allow.Checked = br.ReadBoolean();
                    radioButton_always_forbid.Checked = br.ReadBoolean();
                    radioButton_ask_before.Checked = br.ReadBoolean();
                    // viewer tab
                    textBox_viewer_connect_port.Text = br.ReadString();
                    //server settings tab
                    checkBox_server_enable_invitation_requests.Checked = br.ReadBoolean();
                    textBox_server_listen_connection_via_ip_port.Text = br.ReadString();
                    checkBox_server_set_rdp_port.Checked = br.ReadBoolean();
                    textBox_server_RDP_port.Text = br.ReadString();
                    textBox_server_max_attendees.Text = br.ReadString();
                    radioButton_mirror_driver_attachment_dynamic.Checked = br.ReadBoolean();
                    radioButton_mirror_driver_attachment_static.Checked = br.ReadBoolean();
                    // viewer settings tab
                    checkBox_client_listen_for_invitations.Checked = br.ReadBoolean();
                    textBox_client_listen_connection_via_ip_port.Text = br.ReadString();
                    checkBox_client_set_rpd_port.Checked = br.ReadBoolean();
                    textBox_client_RDP_port.Text = br.ReadString();
                    checkBox_allow_multi_monitor_fullscreen.Checked = br.ReadBoolean();
                    // global settings
                    textBox_user_name.Text = br.ReadString();
                    checkBox_save_settings_on_exit.Checked = br.ReadBoolean();
                }
                catch (EndOfStreamException e)
                {
                    textBox1.Text += Environment.NewLine + "The settings could not be completely loaded. Default values are used.";
                }

                catch (UnauthorizedAccessException)
                {
                    string message = "The settings could not be loaded from the file because of missing permissions. Please run this program as an administrator or change the permissions for this directory.";
                    string caption = "Desktop Sharing 2009";
                    MessageBoxButtons buttons = MessageBoxButtons.OK;


                    // Displays the MessageBox.

                    MessageBox.Show(this, message, caption, buttons,
                    MessageBoxIcon.Error, MessageBoxDefaultButton.Button1);
                }
                catch (IOException e)
                {
                    string message = "The following error occured while loading the settings: " + e.Message;
                    string caption = "Desktop Sharing 2009";
                    MessageBoxButtons buttons = MessageBoxButtons.OK;


                    // Displays the MessageBox.

                    MessageBox.Show(this, message, caption, buttons,
                    MessageBoxIcon.Error, MessageBoxDefaultButton.Button1);
                }
                finally
                {
                    if (br != null)
                    {
                        br.Close();
                        settingsLoaded = true;
                        if (br != null)
                        {
                            ((IDisposable)br).Dispose();
                        }
                    }
                }
            }
        }

        public void startServerSocketListener()
        {
            if (checkBox_server_enable_invitation_requests.Checked)
            {
                try
                {
                    ServerSocketListener = createSocketListener(OnDataReceivedServer, System.Net.IPAddress.Any, Convert.ToInt32(textBox_server_listen_connection_via_ip_port.Text));
                }
                catch (SocketException ex)
                {
                    checkBox_server_enable_invitation_requests.Checked = false;
                    if (ex.ErrorCode == 10048)
                    {
                        textBox1.Text += Environment.NewLine + "Port " + textBox_server_listen_connection_via_ip_port.Text + " is already used by another program. Please choose another port.";
                    }
                    else
                    {
                        textBox1.Text += Environment.NewLine + "Method createSocketListener: The socket could not be created because the following exception occurred: " + ex.ToString();
                    }
                }
            }
        }

        public void startClientSocketListener()
        {
            if (checkBox_client_listen_for_invitations.Checked)
            {
                try
                {
                    ClientSocketListener = createSocketListener(OnDataReceivedViewer, System.Net.IPAddress.Any, Convert.ToInt32(textBox_client_listen_connection_via_ip_port.Text));
                }
                catch (SocketException ex)
                {
                    checkBox_client_listen_for_invitations.Checked = false;
                    if (ex.ErrorCode == 10048)
                    {
                        textBox1.Text += Environment.NewLine + "Port " + textBox_client_listen_connection_via_ip_port.Text + " is already used by another program. Please choose another port.";
                    }
                    else
                    {
                        textBox1.Text += Environment.NewLine + "Method createSocketListener: The socket could not be created because the following exception occurred: " + ex.ToString();
                    }
                }
            }
        }

        public void addParticipantToList(object pObjAttendee)
        {
            IRDPSRAPIAttendee pAttendee = pObjAttendee as IRDPSRAPIAttendee;
            textBox1.Text += Environment.NewLine + ("Attendee connected: " + pAttendee.RemoteName);
            listBox_participants.Items.Add(pAttendee.RemoteName + " (" + pAttendee.Id + ")");
        }

        public void removeParticipantFromList(object pDisconnectInfo)
        {
            IRDPSRAPIAttendeeDisconnectInfo pDiscInfo = pDisconnectInfo as IRDPSRAPIAttendeeDisconnectInfo;
            textBox1.Text += Environment.NewLine + ("Attendee disconnected: " + pDiscInfo.Attendee.RemoteName);
            listBox_participants.Items.Remove(pDiscInfo.Attendee.RemoteName + " (" + pDiscInfo.Attendee.Id + ")");

        }

        public void writeToTextBox(String s)
        {
            textBox1.Text += Environment.NewLine + s;
        }
        //end of my own methods

        private void checkBoxSmartSizing_CheckedChanged(object sender, EventArgs e)
        {
            CheckBox cb = sender as CheckBox;
            Viewer.SmartSizing = cb.Checked;
        }

        private void request_control_button_Click(object sender, EventArgs e)
        {
            try
            {
                Viewer.RequestControl(RDPCOMAPILib.CTRL_LEVEL.CTRL_LEVEL_INTERACTIVE);
            }
            catch (Exception ex)
            {
                string message = "The control request was denied.";
                string caption = "Desktop Sharing 2009";
                MessageBoxButtons buttons = MessageBoxButtons.OK;


                // Displays the MessageBox.

                MessageBox.Show(this, message, caption, buttons,
                MessageBoxIcon.Information, MessageBoxDefaultButton.Button1);
            }
        }

        private void release_control_button_Click(object sender, EventArgs e)
        {
            try
            {
                Viewer.RequestControl(RDPCOMAPILib.CTRL_LEVEL.CTRL_LEVEL_VIEW);
            }
            catch (Exception ex)
            {
                string message = "The control level change request was denied.";
                string caption = "Desktop Sharing 2009";
                MessageBoxButtons buttons = MessageBoxButtons.OK;


                // Displays the MessageBox.

                MessageBox.Show(this, message, caption, buttons,
                MessageBoxIcon.Information, MessageBoxDefaultButton.Button1);
            }
        }



        private void stop_desktop_control_button_Click(object sender, EventArgs e)
        {
            if (SharingSession != null)
            {
                System.Collections.IEnumerator attendees = SharingSession.Attendees.GetEnumerator();

                while (attendees.MoveNext())
                {
                    ((RDPSRAPIAttendee)attendees.Current).ControlLevel = CTRL_LEVEL.CTRL_LEVEL_VIEW;
                    textBox1.Text += Environment.NewLine + "Set control level of attendee " + ((RDPSRAPIAttendee)attendees.Current).Id + " to CTRL_LEVEL_VIEW";
                }
            }
        }

        private void server_set_sharing_rectangle_button_Click(object sender, EventArgs e)
        {
            if (SharingSession != null)
            {
                SharingSession.SetDesktopSharedRect(Convert.ToInt32(textBox_sharing_rectangle_left.Text),
                                                     Convert.ToInt32(textBox_sharing_rectangle_top.Text),
                                                     Convert.ToInt32(textBox_sharing_rectangle_right.Text),
                                                     Convert.ToInt32(textBox_sharing_rectangle_bottom.Text));
            }
        }

        private void server_restore_desktop_sharing_rectangle_button_Click(object sender, EventArgs e)
        {
            if (SharingSession != null)
            {
                SharingSession.SetDesktopSharedRect(SharingSessionRectangle[0], SharingSessionRectangle[1], SharingSessionRectangle[2], SharingSessionRectangle[3]);

                textBox_sharing_rectangle_left.Text = SharingSessionRectangle[0].ToString();
                textBox_sharing_rectangle_top.Text = SharingSessionRectangle[1].ToString();
                textBox_sharing_rectangle_right.Text = SharingSessionRectangle[2].ToString();
                textBox_sharing_rectangle_bottom.Text = SharingSessionRectangle[3].ToString();
            }
        }

        private void checkBox_server_set_rdp_port_CheckedChanged(object sender, EventArgs e)
        {
            CheckBox cb = sender as CheckBox;
            if (cb.Checked)
            {
                if (textBox_server_RDP_port.Text.Length == 0)
                {
                    textBox_server_RDP_port.Text = "5005";
                }
                if (ServerSessionProperties != null)
                {
                    ServerSessionProperties["PortId"] = Convert.ToInt32(textBox_server_RDP_port.Text);
                }
            }
            else
            {
                if (ServerSessionProperties != null)
                {
                    ServerSessionProperties["PortId"] = 0;
                }
            }
        }

        private void checkBox_client_set_rpd_port_CheckedChanged(object sender, EventArgs e)
        {
            CheckBox cb = sender as CheckBox;
            if (cb.Checked)
            {
                if (textBox_client_RDP_port.Text.Length == 0)
                {
                    textBox_client_RDP_port.Text = "5006";
                }
                ClientSessionProperties["PortId"] = Convert.ToInt32(textBox_client_RDP_port.Text);
            }
            else
            {
                ClientSessionProperties["PortId"] = 0;
            }
        }


        private void checkBox_viewer_fullscreen_CheckedChanged(object sender, EventArgs e)
        {
            CheckBox cb = sender as CheckBox;

            if (cb.Checked == true)
            {
                checkBox_viewer_fullscreen_cover_taskbar.Enabled = false;
                numericUpDown_screen_number.Enabled = false;
                numericUpDown_screen_number2.Enabled = false;
                Screen selectedScreen = Screen.AllScreens[Convert.ToInt32(numericUpDown_screen_number.Value) - 1];
                Screen selectedScreen2 = Screen.AllScreens[Convert.ToInt32(numericUpDown_screen_number2.Value) - 1];
                lastViewerWindowLocation = ViewerWindow.Location;
                lastViewerwindowSize = ViewerWindow.Size;
                ViewerWindow.FormBorderStyle = FormBorderStyle.None;

                if (checkBox_allow_multi_monitor_fullscreen.Checked == true)
                {
                    ViewerWindow.Location = new Point(Math.Min(selectedScreen.Bounds.X, selectedScreen2.Bounds.X),
                                                         Math.Min(selectedScreen.Bounds.Y, selectedScreen2.Bounds.Y));
                    ViewerWindow.Size = new Size(selectedScreen.Bounds.Width + selectedScreen2.Bounds.Width,
                                                         Math.Min(selectedScreen.Bounds.Height, selectedScreen2.Bounds.Height));
                }
                else if (checkBox_viewer_fullscreen_cover_taskbar.Checked == true)
                {
                    ViewerWindow.Location = new Point(selectedScreen.Bounds.X, selectedScreen.Bounds.Y);
                    ViewerWindow.Size = new Size(selectedScreen.Bounds.Width, selectedScreen.Bounds.Height);
                }
                else
                {
                    ViewerWindow.Location = new Point(selectedScreen.WorkingArea.X, selectedScreen.WorkingArea.Y);
                    ViewerWindow.Size = new Size(selectedScreen.WorkingArea.Width, selectedScreen.WorkingArea.Height);
                }

                ViewerWindow.TopLevel = true;
            }
            else
            {
                ViewerWindow.FormBorderStyle = FormBorderStyle.Sizable;
                ViewerWindow.Size = lastViewerwindowSize;
                ViewerWindow.Location = lastViewerWindowLocation;
                checkBox_viewer_fullscreen_cover_taskbar.Enabled = true;
                numericUpDown_screen_number.Enabled = true;
                numericUpDown_screen_number2.Enabled = true;
            }
        }

        private void checkBox_allow_multi_monitor_fullscreen_CheckedChanged(object sender, EventArgs e)
        {
            CheckBox cb = sender as CheckBox;

            label_extended_fullscreen.Visible = cb.Checked;
            numericUpDown_screen_number2.Visible = cb.Checked;
        }

        private void numericUpDown_screen_number_ValueChanged(object sender, EventArgs e)
        {
            if (numericUpDown_screen_number.Value > numericUpDown_screen_number2.Value)
            {
                numericUpDown_screen_number2.Value = numericUpDown_screen_number.Value;
            }
        }

        private void numericUpDown_screen_number2_ValueChanged(object sender, EventArgs e)
        {
            if (numericUpDown_screen_number2.Value < numericUpDown_screen_number.Value)
            {
                numericUpDown_screen_number.Value = numericUpDown_screen_number2.Value;
            }
        }

        private void checkBox_server_enable_invitation_requests_CheckedChanged(object sender, EventArgs e)
        {
            if (SharingSession != null)
            {
                CheckBox cb = sender as CheckBox;

                if (cb.Checked == true)
                {
                    startServerSocketListener();
                }
                else
                {
                    try
                    {
                        ServerSocketListener.Close();
                    }
                    catch (ObjectDisposedException) { }
                    catch (NullReferenceException) { }
                    catch (SocketException) { }
                }
            }
        }

        private void checkBox_client_listen_for_invitations_CheckedChanged(object sender, EventArgs e)
        {
            CheckBox cb = sender as CheckBox;

            if (cb.Checked == true)
            {
                startClientSocketListener();
            }
            else
            {
                try
                {
                    ClientSocketListener.Close();
                }
                catch (ObjectDisposedException) { }
                catch (NullReferenceException) { }
                catch (SocketException) { }
            }
        }

        private void button_save_settings_Click(object sender, EventArgs e)
        {
            saveSettings();
        }

        private void MainWindow_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (checkBox_save_settings_on_exit.Checked)
            {
                saveSettings();
            }
        }

        private void contextMenuStrip_participants_ItemClicked(object sender, ToolStripItemClickedEventArgs e)
        {
            if (e.ClickedItem.Text.Equals("Disconnect"))
            {
                foreach (IRDPSRAPIAttendee pAttendee in SharingSession.Attendees)
                {
                    foreach (String item in listBox_participants.SelectedItems)
                    {
                        if (item.Equals(pAttendee.RemoteName + " (" + pAttendee.Id + ")"))
                        {
                            pAttendee.TerminateConnection();
                        }
                    }
                }
            }
            else if (e.ClickedItem.Text.Equals("Show Info"))
            {
                foreach (IRDPSRAPIAttendee pAttendee in SharingSession.Attendees)
                {
                    foreach (String item in listBox_participants.SelectedItems)
                    {
                        if (item.Equals(pAttendee.RemoteName + " (" + pAttendee.Id + ")"))
                        {
                            textBox1.Text += Environment.NewLine + "Remote Name: " + pAttendee.RemoteName;
                            textBox1.Text += Environment.NewLine + "Remote IP address: " + ((IRDPSRAPITcpConnectionInfo)pAttendee.ConnectivityInfo).PeerIP;
                            textBox1.Text += Environment.NewLine + "Remote Port: " + ((IRDPSRAPITcpConnectionInfo)pAttendee.ConnectivityInfo).PeerPort;
                            textBox1.Text += Environment.NewLine + "Local Port: " + ((IRDPSRAPITcpConnectionInfo)pAttendee.ConnectivityInfo).LocalPort;
                        }
                    }
                }
            }
        }

        private void textBox_client_listen_connection_via_ip_port_TextChanged(object sender, EventArgs e)
        {
            if (settingsLoaded & checkBox_client_listen_for_invitations.Checked == true)
            {
                // close current socket
                try
                {
                    ClientSocketListener.Close();
                }
                catch (ObjectDisposedException) { }
                catch (NullReferenceException) { }
                catch (SocketException) { }

                // start new listener socket on new port
                startClientSocketListener();
            }
        }

        private void textBox_server_listen_connection_via_ip_port_TextChanged(object sender, EventArgs e)
        {
            if ((SharingSession != null) & (checkBox_server_enable_invitation_requests.Checked == true))
            {
                // close current socket

                try
                {
                    ServerSocketListener.Close();
                }
                catch (ObjectDisposedException) { }
                catch (NullReferenceException) { }
                catch (SocketException) { }

                // start new listener on new port
                startServerSocketListener();
            }
        }

    }
}
