﻿/*
 * Author: Adrian Posor
 * Year: 2009
 * License: GPL
 */
using System.Windows.Forms;
using System;
using System.Net;
using System.Net.Sockets;

namespace Desktop_Sharing_2009
{
    public partial class MainWindow : Form
    {

        private Socket ServerSocketListener;
        public Socket ClientSocketListener;
        private Socket WorkerSocketServer;
        public Socket WorkerSocketClient;
        private Socket ServerConnectViaIPSocket;
        public Socket ClientConnectViaIPSocket;
        private AsyncCallback WorkerCallbackServer;
        private AsyncCallback WorkerCallbackClient;
        public delegate void FunctionPointer(String str);
        public FunctionPointer write;
        public bool try_reverse_connection = false;
        public String received_connection_string;


        private Socket createSocketListener(AsyncCallback callbackfunction, IPAddress ip, Int32 port)
        {
            Socket socketListener = null;
            //try
            {
                //create the listening socket...
                socketListener = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);
                IPEndPoint ipLocal = new IPEndPoint(ip, port);
                //bind to local IP Address...
                socketListener.Bind(ipLocal);
                //start listening...
                socketListener.Listen(1);
                // create the callback for any client connections...
                ListenerStateObject lso = new ListenerStateObject(callbackfunction, socketListener);
                socketListener.BeginAccept(new AsyncCallback(OnClientConnect), lso);

            }
            //catch (SocketException se)
            //{
                //textBox1.Text += Environment.NewLine + "The socket could not be created because the following exception occurred: " + se.ToString();
            //    this.Invoke(write, new String[] { "The socket could not be created because the following exception occurred: " + se.ToString() });
            //}
            return socketListener;
        }

        public void OnClientConnect(IAsyncResult asyn)
        {
            ListenerStateObject lso = (ListenerStateObject)asyn.AsyncState;
            try
            {
                WaitForData(lso.listener.EndAccept(asyn), lso.workerCallback);
                // be ready for further connections:
                lso.listener.BeginAccept(new AsyncCallback(OnClientConnect), lso);
            }
            catch (ObjectDisposedException ex)
            {
                // This exception usually occurs when we close a listening socket because the OnClientConnect method is called
                // when the listening socket is closed. This is the expected behaviour.
                //textBox1.Text += Environment.NewLine + "OnClientConnect: Socket has been closed. Exception: " + ex.ToString();
            }
            catch (SocketException se)
            {
                //textBox1.Text += Environment.NewLine + "The following exception occured while waiting for data: " + se.ToString();
                this.Invoke(write, new String[] { "The following exception occured while waiting for data: " + se.ToString() });
            }

        }

        private class ListenerStateObject
        {
            public AsyncCallback workerCallback;
            public Socket listener;

            public ListenerStateObject(AsyncCallback wcb, Socket l)
            {
                workerCallback = wcb;
                listener = l;
            }
        }

        private class CSocketPacket
        {
            public System.Net.Sockets.Socket thisSocket;
            public System.Text.StringBuilder sb = new System.Text.StringBuilder();
            public byte[] dataBuffer = new byte[2000];
        }

        public void WaitForData(System.Net.Sockets.Socket soc, AsyncCallback workercallback)
        {
            try
            {
                CSocketPacket theSocPkt = new CSocketPacket();
                theSocPkt.thisSocket = soc;
                // now start to listen for any data...
                soc.BeginReceive(theSocPkt.dataBuffer, 0, theSocPkt.dataBuffer.Length, SocketFlags.None, workercallback, theSocPkt);
            }
            catch (SocketException se)
            {
                if (se.ErrorCode != 10058) // if socket has not been shut down, inform user about error
                {
                    //textBox1.Text += Environment.NewLine + "The following exception occured while waiting for data: " + se.ToString();
                    this.Invoke(write, new String[] { "The following socket exception occured while waiting for data: Error Code: " + se.ErrorCode.ToString() + " " + se.ToString() });
                }
            }
            catch (ObjectDisposedException)
            {
                // we should get here if the socket has been closed only
            }

        }

        public void OnDataReceivedServer(IAsyncResult asyn)
        {
            CSocketPacket theSockId = (CSocketPacket)asyn.AsyncState;

            if (ServerConnectViaIPSocket != null)
            {
                if (!ServerConnectViaIPSocket.Equals(theSockId.thisSocket))
                {
                    WorkerSocketServer = theSockId.thisSocket;
                }
            }
            else
            {
                WorkerSocketServer = theSockId.thisSocket;
            }
            
            //end receive...
            try
            {
                int read = theSockId.thisSocket.EndReceive(asyn);
                theSockId.sb.Append(System.Text.Encoding.ASCII.GetString(theSockId.dataBuffer, 0, read));

                if (theSockId.sb.ToString().Equals("RI"))
                {
                    try
                    {
                        // The viewer has requested an invitation, so we send one to him.
                        theSockId.thisSocket.Send(System.Text.Encoding.ASCII.GetBytes(invitation_from_server.ConnectionString));
                    }
                    catch (Exception ex)
                    {
                        //textBox1.Text += Environment.NewLine + "The server could not send the invitation to the client because the following exception occurred: " + ex.ToString();
                        this.Invoke(write, new String[] { "The server could not send the invitation to the client because the following exception occurred: " + ex.ToString() });
                    }
                    finally
                    {
                        // be ready to receive further data, but use an empty receive buffer
                        WaitForData(theSockId.thisSocket, OnDataReceivedServer);
                    }
                }
                else if (theSockId.sb.ToString().Trim().StartsWith("<E>"))
                {
                    if (theSockId.sb.ToString().Trim().EndsWith("</E>"))
                    {
                        this.Invoke(write, new String[] { "RDP Session: Connecting to client..." });
                        try
                        {
                            SharingSession.ConnectToClient(theSockId.sb.ToString());
                        }
                        catch (Exception ex)
                        {
                            //textBox1.Text += Environment.NewLine + "The server could not connect to the client because the following exception occurred: " + ex.ToString();
                            this.Invoke(write, new String[] { "The server could not connect to the client because the following exception occurred: " + ex.ToString() });
                        }
                        finally
                        {
                            // be ready to receive further data, but use an empty receive buffer
                            WaitForData(theSockId.thisSocket, OnDataReceivedServer);
                        }
                    }
                    else
                    {
                        // Invitation has not been completely received. Wait for the rest of the message.
                        //WaitForData(theSockId.thisSocket, OnDataReceivedServer);
                        theSockId.thisSocket.BeginReceive(theSockId.dataBuffer, 0, theSockId.dataBuffer.Length, SocketFlags.None, OnDataReceivedServer, theSockId);
                    }
                }
                else
                {
                    // be ready to receive further data, but use an empty receive buffer
                    // we discard any data received so far
                    WaitForData(theSockId.thisSocket, OnDataReceivedServer);
                }
            }
            catch (ObjectDisposedException)
            {
                // we should only get here if the socket is closed
            }
            catch (SocketException ex)
            {
                textBox1.Text += Environment.NewLine + ex.Message;
            }
        }

        public void OnDataReceivedViewer(IAsyncResult asyn)
        {
            //bool try_reverse_connection = false;
            CSocketPacket theSockId = (CSocketPacket)asyn.AsyncState;

            if (ClientConnectViaIPSocket != null)
            {
                if (!ClientConnectViaIPSocket.Equals(theSockId.thisSocket))
                {
                    WorkerSocketClient = theSockId.thisSocket;
                }
            }
            else
            {
                WorkerSocketClient = theSockId.thisSocket;
            }
            
            //end receive...
            try
            {
                int read = theSockId.thisSocket.EndReceive(asyn);
                theSockId.sb.Append(System.Text.Encoding.ASCII.GetString(theSockId.dataBuffer, 0, read));


                if (theSockId.sb.ToString().Trim().StartsWith("<E>"))
                {
                    if (theSockId.sb.ToString().Trim().EndsWith("</E>"))
                    {
                        received_connection_string = theSockId.sb.ToString();
                        try
                        {
                            // set flag for "OnConnectionFailed" routine
                            try_reverse_connection = true;

                            client_connect(received_connection_string, false);
                        }
                        catch (Exception ex)
                        {
                            // The event "OnConnectionFailed" appears if the connection cannot be established.
                            // No exception is thrown and the connect method has no return value either.
                            //textBox1.Text += Environment.NewLine + "The client could not connect to the server.";
                            this.Invoke(write, new String[] { "The client could not connect to the server." });
                            //textBox1.Text += Environment.NewLine + "Trying reverse-connection...";
                            this.Invoke(write, new String[] { "Trying reverse-connection..." });
                            try_reverse_connection = true;
                        }
                        finally
                        {
                            // be ready to receive further data, but use an empty receive buffer
                            WaitForData(theSockId.thisSocket, OnDataReceivedServer);
                        }
                    }
                    else
                    {
                        // Invitation has not been completely received. Wait for the rest of the message.
                        //WaitForData(theSockId.thisSocket, OnDataReceivedServer);
                        theSockId.thisSocket.BeginReceive(theSockId.dataBuffer, 0, theSockId.dataBuffer.Length, SocketFlags.None, OnDataReceivedViewer, theSockId);
                    }
                }
                else
                {
                    // be ready to receive further data, but use an empty receive buffer
                    WaitForData(theSockId.thisSocket, OnDataReceivedServer);
                }

                //if (try_reverse_connection == true)
                //{
                //    String reverse_connection_string;
                //    startViewer();
                //    reverse_connection_string = start_client_listener();

                //    try
                //    {
                //        theSockId.thisSocket.Send(System.Text.Encoding.ASCII.GetBytes(reverse_connection_string));
                //    }
                //    catch (Exception ex)
                //    {
                //        textBox1.Text += Environment.NewLine + "The client could not initiate a reverse-connection because the following exception occurred: " + ex.ToString();
                //    }
                //}
            }
            catch (ObjectDisposedException)
            {
                // we should only get here if the socket is closed
            }
            catch (SocketException ex)
            {
                textBox1.Text += Environment.NewLine + ex.Message;
            }
        }

        public void initiateReverseConnection()
        {
            String reverse_connection_string;
            startViewer();
            reverse_connection_string = start_client_listener(received_connection_string);

            try
            {
                WorkerSocketClient.Send(System.Text.Encoding.ASCII.GetBytes(reverse_connection_string));
            }
            catch (Exception ex)
            {
                //textBox1.Text += Environment.NewLine + "The client could not initiate a reverse-connection because the following exception occurred: " + ex.ToString();
                this.Invoke(write, new String[] { "The client could not initiate a reverse-connection because the following exception occurred: " + ex.ToString() });
            }
        }
    }

}
